@@ -2,10 +2,19 @@ use super::{ObjectError, ObjectId, Result, WriteObject};
 use crate::{
     backends::Backend,
     compress,
-    crypto::{ChunkKey, CryptoProvider, Digest},
+    crypto::{ChunkKey, CryptoProvider, Digest, RootKey},
     ChunkPointer,
 };
-use std::sync::Arc;
+use std::{
+    io::{Seek, SeekFrom},
+    sync::Arc,
+};
+
+#[derive(Clone)]
+enum Mode {
+    SealRoot(u64),
+    Data,
+}
 
 pub trait Writer: Send {
     fn write_chunk(&mut self, hash: &Digest, data: &[u8]) -> Result<ChunkPointer>;
@@ -16,6 +25,7 @@ pub struct AEADWriter {
     backend: Arc<dyn Backend>,
     crypto: ChunkKey,
     object: WriteObject,
+    mode: Mode,
 }
 
 impl AEADWriter {
@@ -27,8 +37,37 @@ impl AEADWriter {
             backend,
             crypto,
             object,
+            mode: Mode::Data,
         }
     }
+
+    pub fn for_root(backend: Arc<dyn Backend>, crypto: RootKey, header_size: u64) -> Self {
+        let mut object = WriteObject::default();
+        object.reset_id(&crypto);
+        object.seek(SeekFrom::Start(header_size)).unwrap();
+
+        AEADWriter {
+            backend,
+            crypto,
+            object,
+            mode: Mode::SealRoot(header_size),
+        }
+    }
+
+    pub(crate) fn flush_root_head(&mut self, id: ObjectId, head: &[u8]) -> Result<()> {
+        self.object.set_id(id);
+        self.object.write_head(head);
+        self.object.finalize(&self.crypto);
+        self.backend.write_object(&self.object)?;
+
+        self.object.reset_id(&self.crypto);
+        self.object.seek(match self.mode {
+            Mode::SealRoot(header) => SeekFrom::Start(header),
+            Mode::Data => SeekFrom::Start(0),
+        })?;
+
+        Ok(())
+    }
 }
 
 impl Clone for AEADWriter {
@@ -40,6 +79,7 @@ impl Clone for AEADWriter {
             object,
             backend: self.backend.clone(),
             crypto: self.crypto.clone(),
+            mode: self.mode.clone(),
         }
     }
 }
@@ -74,25 +114,39 @@ impl Writer for AEADWriter {
         };
 
         let oid = *self.object.id();
-        let (size, tag) = {
+        let mut pointer = {
             let buffer = self.object.tail_mut();
-            let tag = self.crypto.encrypt_chunk(&oid, hash, &mut buffer[..size]);
 
-            (size, tag)
+            self.crypto.encrypt_chunk(
+                match self.mode {
+                    Mode::Data => Some(oid),
+                    Mode::SealRoot(_) => None,
+                },
+                hash,
+                &mut buffer[..size],
+            )
         };
 
-        let offs = self.object.position();
-        *self.object.position_mut() += size;
+        pointer.file = oid;
+        pointer.offs = self.object.position() as u32;
+        *self.object.position_mut() += pointer.size as usize;
 
-        Ok(ChunkPointer::new(offs as u32, size as u32, oid, *hash, tag))
+        Ok(pointer.into())
     }
 
     fn flush(&mut self) -> Result<()> {
+        if let Mode::SealRoot(header_size) = self.mode {
+            self.object
+                .randomize_head(&self.crypto, header_size.try_into().unwrap());
+        }
         self.object.finalize(&self.crypto);
         self.backend.write_object(&self.object)?;
 
         self.object.reset_id(&self.crypto);
-        self.object.reset_cursor();
+        self.object.seek(match self.mode {
+            Mode::SealRoot(header) => SeekFrom::Start(header),
+            Mode::Data => SeekFrom::Start(0),
+        })?;
 
         Ok(())
     }
